clear;
close all;
%Aufgabe 1: Puma Roboter:
mdl_puma560 %Roboter Laden
q0 = [0 0 0 0 0 0];  %Nullstellung aller Gelenke
p560.teach(q0);     %Anzeigen des Roboters
T1 = p560.fkine(q0);    %Forwrtskinematik fr die Nullpose beechnen
T2 =SE3(0.7,0.3,0)*SE3.Ry(pi/2);    %Zielpunkt des Roboters

IV2 = p560.ikine(T2);   %Inverse Kinematik des Zielpunkts
p560.plot(IV2)

t1 = [0:0.05:2]';   %Zeitvektor fr die Trajektorie
[q qd qdd] = mtraj(@lspb,q0,IV2,t1);    %Erstellen der Trajektorie
p560.plot(q)


%Aufgabe 2: Modellieren des Roboterarm aus der letzen bung
%%% Erstellen des Roboters
clear;
close all;
prismatic_joint = Prismatic( ...
	'theta', pi/2, ...
	'a', 0, ...
	'alpha', pi/2, ...
	...
	'm', 1, ...
	'r', [0 0 -0.5], ...
	'I', [0.25 0.25 0], ...
	'B', 0, ...
	'G', 0, ...
	'Jm', 0, ...
	'standard')

revolute_joint = Revolute( ...
	'd', 0,  ...
	'a', 1,  ...
	'alpha', 0,  ...
	...
	'm', 1, ...
	'r', [-0.5 0 0],  ... 
	'I', [0.25 0.25 0],  ...
	'B', 0,  ...
	'G', 0,  ...
	'Jm', 0,  ...
	'standard')

two_link = SerialLink([
    prismatic_joint
    revolute_joint
    ]);
two_link.base=SE3(0,0,0) * SE3.Ry(pi/2);
two_link.links(2).offset = pi/2;
two_link.links(1).offset = 1;
two_link.links(1).qlim = [0 0.5];
two_link.gravity=([0 0 9.81]);

%%% Anzeige des Roboters
ws = [0 3 -1 1 -0.5 3];    %Size of Workspace
plot_options = {'workspace',ws};
two_link.plotopt = plot_options;
two_link.teach([0 0], 'view','top');


%%Trajectory in joint space

FK = two_link.fkine([0.5 pi/2])  % Vorwrtskinematik
FK.toeul %Eulerwinkel 

%%Mask: Genutzte Robotergelenke in kartesischen Koordinaten im Endeffektorkoordinatensystems.
%%Hier bewegung in XY Ebene 
IK = two_link.ikine(FK,'mask',[1 1 0 0 0 0]);

%q0: Anfangswerte fr IK
T1 = transl(2,0,0);  %Startpunkt der Translation
T2 = transl(1.5,1,0)*trotz(pi/2);  %Endpunkt der Translation
%%Inverse Kinematik des Start und Zielpunktes
q1 = two_link.ikine(T1,'mask',[1 1 0 0 0 0]);
q2 = two_link.ikine(T2,'mask',[1 1 0 0 0 0]);
%% Zeitvektor
t1 = [0:0.05:2]';
%%Trajektorie im Jointspace (Standartmig mit Polynomapprox.)
[q qd qdd] = mtraj(@lspb,q1,q2,t1);
%%Plotten der Trajektorie
two_link.plot(q)
hold on
%%Berechnen und Plotten der Endeffektorbewegung
FK1 = two_link.fkine(q);
p = transl(FK1);
plot3(p(:,1),p(:,2),p(:,3));
%%Plotten der Jointtrajektorien
figure(2)
plot(t1,q);
legend('q1','q2');

%%Jacobi Matrix
J1 = two_link.jacob0([0 0]);
figure(3);
two_link.plot([ 0 0], 'view','top');
%%In diesem Fall dreht sich das Gelenk nicht in positive Richtung. D.h. Drehung
%%um die Z Achse und Translation in negative X und Y Richtung
vB1 = J1*[0.5 0]';
   
J3 = two_link.jacob0([0 pi/4]);
vB3 = J3*[0 pi/2]';
two_link.plot([ 0 pi/4], 'view','top');

%%Recursive newton Euler Verfahren zum Berechnen der Krfte und Momente in
%%den Gelenken

qn = [0 0];
qz = [0 0];
%%Berechnen der Joint Torque/Force fr Position 0 0.
Q = two_link.rne(qn,qz,qz);

%Transpose Jacobi
Fend = [ 3 3 0 0 0 2];       %%Kraftvektor am Ende des Arms
F = J1'*Fend';  %% Kraft in den Gelenken erzeugt durch Krfte und Momente am Ende des Arms fr Stellung [0 0];

%Aufgabe 3: Regelung
clear;
close all;

% Charakteristische Eigenschaften des Motors
Rm=0.235; % Stator phase resistance Rs (ohm) 
Lm=0.0008; % Armature inductance (H) 
Km_torque=0.11; % Torque constant [N*m/A_peak] 
%bertragungsfunktion des Motors
motor =tf([1],[Lm Rm]);

%Charakteristische Eigenschaften des Arms
InertiaArm=5.3933e-05;  %%[kg*m^2] 
ViscFr=0.00074; % Viscous friction coefficients at base (B) [N*m/(rad/s )] 
%bertragungsfunktion des Arms
robotarm = tf([1],[InertiaArm ViscFr]);

%Trajektorie fr den Roboterarm
qdv = [t1 qd(:,2)];
